@@ -233,6 +233,8 @@ CitusCopyFrom(CopyStmt *copyStatement, char *completionTag)
 							errmsg("unsupported partition method")));
 		}
 	}
+
+	XactModificationLevel = XACT_MODIFICATION_DATA;
 }
 
 
@@ -268,6 +270,14 @@ CopyFromWorkerNode(CopyStmt *copyStatement, char *completionTag)
 	int32 nodePort = masterNodeAddress->nodePort;
 	char *nodeUser = CurrentUserName();
 
+	if (XactModificationLevel > XACT_MODIFICATION_NONE)
+	{
+		ereport(ERROR, (errcode(ERRCODE_ACTIVE_SQL_TRANSACTION),
+						errmsg("distributed copy operations must not appear in "
+							   "transaction blocks containing other distributed "
+							   "modifications")));
+	}
+
 	masterConnection = ConnectToNode(nodeName, nodePort, nodeUser);
 
 	PG_TRY();
@@ -361,7 +371,7 @@ CopyToExistingShards(CopyStmt *copyStatement, char *completionTag)
 	ShardInterval **shardIntervalCache = NULL;
 	bool useBinarySearch = false;
 
-	HTAB *shardConnectionHash = NULL;
+	HTAB *copyConnectionHash = NULL;
 	ShardConnections *shardConnections = NULL;
 	List *connectionList = NIL;
 
@@ -423,7 +433,8 @@ CopyToExistingShards(CopyStmt *copyStatement, char *completionTag)
 	}
 
 	/* prevent concurrent placement changes and non-commutative DML statements */
-	LockShards(shardIntervalList, ShareLock);
+	LockShardListMetadata(shardIntervalList, ShareLock);
+	LockShardListResources(shardIntervalList, ShareLock);
 
 	/* initialize the shard interval cache */
 	shardCount = cacheEntry->shardIntervalArrayLength;
@@ -463,7 +474,7 @@ CopyToExistingShards(CopyStmt *copyStatement, char *completionTag)
 	 * PG_CATCH. Otherwise, it may be undefined in the PG_CATCH (see sigsetjmp
 	 * documentation).
 	 */
-	shardConnectionHash = CreateShardConnectionHash();
+	copyConnectionHash = CreateShardConnectionHash(TopTransactionContext);
 
 	/* we use a PG_TRY block to roll back on errors (e.g. in NextCopyFrom) */
 	PG_TRY();
@@ -532,9 +543,8 @@ CopyToExistingShards(CopyStmt *copyStatement, char *completionTag)
 			MemoryContextSwitchTo(oldContext);
 
 			/* get existing connections to the shard placements, if any */
-			shardConnections = GetShardConnections(shardConnectionHash,
-												   shardId,
-												   &shardConnectionsFound);
+			shardConnections = GetShardHashConnections(copyConnectionHash, shardId,
+													   &shardConnectionsFound);
 			if (!shardConnectionsFound)
 			{
 				/* open connections and initiate COPY on shard placements */
@@ -558,7 +568,7 @@ CopyToExistingShards(CopyStmt *copyStatement, char *completionTag)
 			processedRowCount += 1;
 		}
 
-		connectionList = ConnectionList(shardConnectionHash);
+		connectionList = ConnectionList(copyConnectionHash);
 
 		/* send copy binary footers to all shard placements */
 		if (copyOutState->binary)
@@ -588,7 +598,7 @@ CopyToExistingShards(CopyStmt *copyStatement, char *completionTag)
 		List *abortConnectionList = NIL;
 
 		/* roll back all transactions */
-		abortConnectionList = ConnectionList(shardConnectionHash);
+		abortConnectionList = ConnectionList(copyConnectionHash);
 		EndRemoteCopy(abortConnectionList, false);
 		AbortRemoteTransactions(abortConnectionList);
 		CloseConnections(abortConnectionList);
@@ -934,11 +944,21 @@ OpenCopyTransactions(CopyStmt *copyStatement, ShardConnections *shardConnections
 
 	MemoryContextSwitchTo(oldContext);
 
+	if (XactModificationLevel > XACT_MODIFICATION_NONE)
+	{
+		ereport(ERROR, (errcode(ERRCODE_ACTIVE_SQL_TRANSACTION),
+						errmsg("distributed copy operations must not appear in "
+							   "transaction blocks containing other distributed "
+							   "modifications")));
+	}
+
 	foreach(placementCell, finalizedPlacementList)
 	{
 		ShardPlacement *placement = (ShardPlacement *) lfirst(placementCell);
 		char *nodeName = placement->nodeName;
 		int nodePort = placement->nodePort;
+		WorkerNode *workerNode = FindWorkerNode(nodeName, nodePort);
+		int workerGroupId = 0;
 		char *nodeUser = CurrentUserName();
 		PGconn *connection = ConnectToNode(nodeName, nodePort, nodeUser);
 
@@ -946,6 +966,17 @@ OpenCopyTransactions(CopyStmt *copyStatement, ShardConnections *shardConnections
 		StringInfo copyCommand = NULL;
 		PGresult *result = NULL;
 
+		/*
+		 * When a copy is initiated from a worker, the information about the connected
+		 * worker node may not be found if pg_dist_node entries are not synced to this
+		 * node. In that case we leave the groupId as 0. Fortunately, it is unused since
+		 * COPY from a worker does not initiate a 2PC.
+		 */
+		if (workerNode != NULL)
+		{
+			workerGroupId = workerNode->groupId;
+		}
+
 		if (connection == NULL)
 		{
 			if (stopOnFailure)
@@ -986,9 +1017,12 @@ OpenCopyTransactions(CopyStmt *copyStatement, ShardConnections *shardConnections
 
 		transactionConnection = palloc0(sizeof(TransactionConnection));
 
+		transactionConnection->groupId = workerGroupId;
 		transactionConnection->connectionId = shardConnections->shardId;
 		transactionConnection->transactionState = TRANSACTION_STATE_COPY_STARTED;
 		transactionConnection->connection = connection;
+		transactionConnection->nodeName = nodeName;
+		transactionConnection->nodePort = nodePort;
 
 		connectionList = lappend(connectionList, transactionConnection);
 	}
@@ -1009,12 +1043,8 @@ OpenCopyTransactions(CopyStmt *copyStatement, ShardConnections *shardConnections
 	foreach(failedPlacementCell, failedPlacementList)
 	{
 		ShardPlacement *failedPlacement = (ShardPlacement *) lfirst(failedPlacementCell);
-		uint64 shardLength = 0;
 
-		DeleteShardPlacementRow(failedPlacement->shardId, failedPlacement->nodeName,
-								failedPlacement->nodePort);
-		InsertShardPlacementRow(failedPlacement->shardId, FILE_INACTIVE, shardLength,
-								failedPlacement->nodeName, failedPlacement->nodePort);
+		UpdateShardPlacementState(failedPlacement->placementId, FILE_INACTIVE);
 	}
 
 	shardConnections->connectionList = connectionList;
