@@ -94,7 +94,7 @@ MultiClientConnect(const char *nodeName, uint32 nodePort, const char *nodeDataba
 	char *effectiveDatabaseName = NULL;
 	char *effectiveUserName = NULL;
 
-	if (IsModifyingTransaction)
+	if (XactModificationLevel > XACT_MODIFICATION_NONE)
 	{
 		ereport(ERROR, (errcode(ERRCODE_ACTIVE_SQL_TRANSACTION),
 						errmsg("cannot open new connections after the first modification "
@@ -181,7 +181,7 @@ MultiClientConnectStart(const char *nodeName, uint32 nodePort, const char *nodeD
 		return connectionId;
 	}
 
-	if (IsModifyingTransaction)
+	if (XactModificationLevel > XACT_MODIFICATION_NONE)
 	{
 		ereport(ERROR, (errcode(ERRCODE_ACTIVE_SQL_TRANSACTION),
 						errmsg("cannot open new connections after the first modification "
@@ -469,6 +469,8 @@ MultiClientQueryResult(int32 connectionId, void **queryResult, int *rowCount,
 	{
 		WarnRemoteError(connection, result);
 		PQclear(result);
+
+		return false;
 	}
 
 	/* clear extra result objects */
@@ -881,7 +883,7 @@ MultiClientWait(WaitInfo *waitInfo)
 		}
 		else if (rc == 0)
 		{
-			ereport(DEBUG2,
+			ereport(DEBUG5,
 					(errmsg("waiting for activity on tasks took longer than %ld ms",
 							(long) RemoteTaskCheckInterval * 10)));
 		}
