@@ -24,7 +24,6 @@
 #include "access/xact.h"
 #include "catalog/namespace.h"
 #include "commands/dbcommands.h"
-#include "distributed/master_metadata_utility.h"
 #include "distributed/master_protocol.h"
 #include "distributed/multi_client_executor.h"
 #include "distributed/multi_join_order.h"
@@ -61,8 +60,6 @@ static List * ShardsMatchingDeleteCriteria(Oid relationId, List *shardList,
 										   Node *deleteCriteria);
 static int DropShards(Oid relationId, char *schemaName, char *relationName,
 					  List *deletableShardIntervalList);
-static bool ExecuteRemoteCommand(const char *nodeName, uint32 nodePort,
-								 StringInfo queryString);
 
 
 /* exports for SQL callable functions */
@@ -319,25 +316,14 @@ DropShards(Oid relationId, char *schemaName, char *relationName,
 		ListCell *lingeringPlacementCell = NULL;
 		ShardInterval *shardInterval = (ShardInterval *) lfirst(shardIntervalCell);
 		uint64 shardId = shardInterval->shardId;
-		char *shardAlias = NULL;
 		char *quotedShardName = NULL;
-		StringInfo shardName = makeStringInfo();
+		char *shardRelationName = pstrdup(relationName);
 
 		Assert(shardInterval->relationId == relationId);
 
-		/* if shard doesn't have an alias, extend regular table name */
-		shardAlias = LoadShardAlias(relationId, shardId);
-		if (shardAlias == NULL)
-		{
-			appendStringInfoString(shardName, relationName);
-			AppendShardIdToStringInfo(shardName, shardId);
-		}
-		else
-		{
-			appendStringInfoString(shardName, shardAlias);
-		}
-
-		quotedShardName = quote_qualified_identifier(schemaName, shardName->data);
+		/* Build shard relation name. */
+		AppendShardIdToName(&shardRelationName, shardId);
+		quotedShardName = quote_qualified_identifier(schemaName, shardRelationName);
 
 		shardPlacementList = ShardPlacementList(shardId);
 		foreach(shardPlacementCell, shardPlacementList)
@@ -390,16 +376,17 @@ DropShards(Oid relationId, char *schemaName, char *relationName,
 		foreach(lingeringPlacementCell, lingeringPlacementList)
 		{
 			ShardPlacement *placement = (ShardPlacement *) lfirst(lingeringPlacementCell);
+			uint64 placementId = placement->placementId;
 			char *workerName = placement->nodeName;
 			uint32 workerPort = placement->nodePort;
 			uint64 oldShardLength = placement->shardLength;
 
 			DeleteShardPlacementRow(shardId, workerName, workerPort);
-			InsertShardPlacementRow(shardId, FILE_TO_DELETE, oldShardLength,
+			InsertShardPlacementRow(shardId, placementId, FILE_TO_DELETE, oldShardLength,
 									workerName, workerPort);
 
 			ereport(WARNING, (errmsg("could not delete shard \"%s\" on node \"%s:%u\"",
-									 shardName->data, workerName, workerPort),
+									 shardRelationName, workerName, workerPort),
 							  errdetail("Marking this shard placement for deletion")));
 		}
 
@@ -567,62 +554,3 @@ ShardsMatchingDeleteCriteria(Oid relationId, List *shardIntervalList,
 
 	return dropShardIntervalList;
 }
-
-
-/*
- * ExecuteRemoteCommand executes the given SQL command. This command could be an
- * Insert, Update, or Delete statement, or a utility command that returns
- * nothing. If query is successfuly executed, the function returns true.
- * Otherwise, it returns false.
- */
-static bool
-ExecuteRemoteCommand(const char *nodeName, uint32 nodePort, StringInfo queryString)
-{
-	char *nodeDatabase = get_database_name(MyDatabaseId);
-	int32 connectionId = -1;
-	QueryStatus queryStatus = CLIENT_INVALID_QUERY;
-	bool querySent = false;
-	bool queryReady = false;
-	bool queryDone = false;
-
-	connectionId = MultiClientConnect(nodeName, nodePort, nodeDatabase, NULL);
-	if (connectionId == INVALID_CONNECTION_ID)
-	{
-		return false;
-	}
-
-	querySent = MultiClientSendQuery(connectionId, queryString->data);
-	if (!querySent)
-	{
-		MultiClientDisconnect(connectionId);
-		return false;
-	}
-
-	while (!queryReady)
-	{
-		ResultStatus resultStatus = MultiClientResultStatus(connectionId);
-		if (resultStatus == CLIENT_RESULT_READY)
-		{
-			queryReady = true;
-		}
-		else if (resultStatus == CLIENT_RESULT_BUSY)
-		{
-			long sleepIntervalPerCycle = RemoteTaskCheckInterval * 1000L;
-			pg_usleep(sleepIntervalPerCycle);
-		}
-		else
-		{
-			MultiClientDisconnect(connectionId);
-			return false;
-		}
-	}
-
-	queryStatus = MultiClientQueryStatus(connectionId);
-	if (queryStatus == CLIENT_QUERY_DONE)
-	{
-		queryDone = true;
-	}
-
-	MultiClientDisconnect(connectionId);
-	return queryDone;
-}
