@@ -20,12 +20,14 @@
 #include "catalog/pg_type.h"
 #include "distributed/citus_nodes.h"
 #include "distributed/master_metadata_utility.h"
+#include "distributed/master_protocol.h"
 #include "distributed/metadata_cache.h"
 #include "distributed/multi_join_order.h"
 #include "distributed/multi_logical_optimizer.h"
 #include "distributed/pg_dist_partition.h"
 #include "distributed/pg_dist_shard.h"
 #include "distributed/pg_dist_shard_placement.h"
+#include "distributed/relay_utility.h"
 #include "distributed/worker_manager.h"
 #include "nodes/makefuncs.h"
 #include "parser/scansup.h"
@@ -50,6 +52,61 @@ PG_FUNCTION_INFO_V1(master_stage_shard_row);
 PG_FUNCTION_INFO_V1(master_stage_shard_placement_row);
 
 
+/*
+ * TableShardReplicationFactor returns the current replication factor of the
+ * given relation by looking into shard placements. It errors out if there
+ * are different number of shard placements for different shards. It also
+ * errors out if the table does not have any shards.
+ */
+uint32
+TableShardReplicationFactor(Oid relationId)
+{
+	uint32 replicationCount = 0;
+	ListCell *shardCell = NULL;
+
+	List *shardIntervalList = LoadShardIntervalList(relationId);
+	foreach(shardCell, shardIntervalList)
+	{
+		ShardInterval *shardInterval = (ShardInterval *) lfirst(shardCell);
+		uint64 shardId = shardInterval->shardId;
+
+		List *shardPlacementList = ShardPlacementList(shardId);
+		uint32 shardPlacementCount = list_length(shardPlacementList);
+
+		/*
+		 * Get the replication count of the first shard in the list, and error
+		 * out if there is a shard with different replication count.
+		 */
+		if (replicationCount == 0)
+		{
+			replicationCount = shardPlacementCount;
+		}
+		else if (replicationCount != shardPlacementCount)
+		{
+			char *relationName = get_rel_name(relationId);
+			ereport(ERROR, (errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
+							errmsg("cannot find the replication factor of the "
+								   "table %s", relationName),
+							errdetail("The shard %ld has different shards replication "
+									  "counts from other shards.", shardId)));
+		}
+	}
+
+	/* error out if the table does not have any shards */
+	if (replicationCount == 0)
+	{
+		char *relationName = get_rel_name(relationId);
+		ereport(ERROR, (errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
+						errmsg("cannot find the replication factor of the "
+							   "table %s", relationName),
+						errdetail("The table %s does not have any shards.",
+								  relationName)));
+	}
+
+	return replicationCount;
+}
+
+
 /*
  * LoadShardIntervalList returns a list of shard intervals related for a given
  * distributed table. The function returns an empty list if no shards can be
@@ -138,72 +195,6 @@ AllocateUint64(uint64 value)
 }
 
 
-/*
- * LoadShardAlias finds the row for given relation and shardId in pg_dist_shard,
- * finds the shard alias in this row if any, and then deep copies this alias.
- */
-char *
-LoadShardAlias(Oid relationId, uint64 shardId)
-{
-	SysScanDesc scanDescriptor = NULL;
-	ScanKeyData scanKey[1];
-	int scanKeyCount = 1;
-	HeapTuple heapTuple = NULL;
-	Datum shardAliasDatum = 0;
-	bool shardAliasNull = false;
-	char *shardAlias = NULL;
-
-	Relation pgDistShard = heap_open(DistShardRelationId(), AccessShareLock);
-	TupleDesc tupleDescriptor = RelationGetDescr(pgDistShard);
-
-	ScanKeyInit(&scanKey[0], Anum_pg_dist_shard_shardid,
-				BTEqualStrategyNumber, F_INT8EQ, Int64GetDatum(shardId));
-
-	scanDescriptor = systable_beginscan(pgDistShard,
-										DistShardShardidIndexId(), true,
-										NULL, scanKeyCount, scanKey);
-
-	/*
-	 * Normally, we should have at most one tuple here as we have a unique index
-	 * on shardId. However, if users want to drop this uniqueness constraint,
-	 * and look up the shardalias based on the relation and shardId pair, we
-	 * still allow that. We don't have any users relaying on this feature. Thus,
-	 * we may consider to remove this check.
-	 */
-	heapTuple = systable_getnext(scanDescriptor);
-	while (HeapTupleIsValid(heapTuple))
-	{
-		Form_pg_dist_shard pgDistShardForm = (Form_pg_dist_shard) GETSTRUCT(heapTuple);
-		if (pgDistShardForm->logicalrelid == relationId)
-		{
-			break;
-		}
-
-		heapTuple = systable_getnext(scanDescriptor);
-	}
-
-	/* if no tuple found, error out */
-	if (!HeapTupleIsValid(heapTuple))
-	{
-		ereport(ERROR, (errmsg("could not find valid entry for relationId: %u "
-							   "and shard " UINT64_FORMAT, relationId, shardId)));
-	}
-
-	/* if shard alias exists, deep copy cstring */
-	shardAliasDatum = heap_getattr(heapTuple, Anum_pg_dist_shard_shardalias,
-								   tupleDescriptor, &shardAliasNull);
-	if (!shardAliasNull)
-	{
-		shardAlias = TextDatumGetCString(shardAliasDatum);
-	}
-
-	systable_endscan(scanDescriptor);
-	heap_close(pgDistShard, AccessShareLock);
-
-	return shardAlias;
-}
-
-
 /*
  * CopyShardInterval copies fields from the specified source ShardInterval
  * into the fields of the provided destination ShardInterval.
@@ -265,6 +256,45 @@ ShardLength(uint64 shardId)
 }
 
 
+/*
+ * NodeHasActiveShardPlacements returns whether any active shards are placed on this node
+ */
+bool
+NodeHasActiveShardPlacements(char *nodeName, int32 nodePort)
+{
+	const int scanKeyCount = 3;
+	const bool indexOK = false;
+
+	bool hasFinalizedPlacements = false;
+
+	HeapTuple heapTuple = NULL;
+	SysScanDesc scanDescriptor = NULL;
+	ScanKeyData scanKey[scanKeyCount];
+
+	Relation pgShardPlacement = heap_open(DistShardPlacementRelationId(),
+										  AccessShareLock);
+
+	ScanKeyInit(&scanKey[0], Anum_pg_dist_shard_placement_nodename,
+				BTEqualStrategyNumber, F_TEXTEQ, CStringGetTextDatum(nodeName));
+	ScanKeyInit(&scanKey[1], Anum_pg_dist_shard_placement_nodeport,
+				BTEqualStrategyNumber, F_INT8EQ, Int64GetDatum(nodePort));
+	ScanKeyInit(&scanKey[2], Anum_pg_dist_shard_placement_shardstate,
+				BTEqualStrategyNumber, F_INT4EQ, Int32GetDatum(FILE_FINALIZED));
+
+	scanDescriptor = systable_beginscan(pgShardPlacement,
+										DistShardPlacementNodeidIndexId(), indexOK,
+										NULL, scanKeyCount, scanKey);
+
+	heapTuple = systable_getnext(scanDescriptor);
+	hasFinalizedPlacements = HeapTupleIsValid(heapTuple);
+
+	systable_endscan(scanDescriptor);
+	heap_close(pgShardPlacement, AccessShareLock);
+
+	return hasFinalizedPlacements;
+}
+
+
 /*
  * FinalizedShardPlacementList finds shard placements for the given shardId from
  * system catalogs, chooses placements that are in finalized state, and returns
@@ -341,9 +371,9 @@ ShardPlacementList(uint64 shardId)
 
 
 /*
- * TupleToShardPlacement takes in a heap tuple from pg_dist_shard_placement, and
- * converts this tuple to an equivalent struct in memory. The function assumes
- * the caller already has locks on the tuple, and doesn't perform any locking.
+ * TupleToShardPlacement takes in a heap tuple from pg_dist_shard_placement,
+ * and converts this tuple to in-memory struct. The function assumes the
+ * caller already has locks on the tuple, and doesn't perform any locking.
  */
 ShardPlacement *
 TupleToShardPlacement(TupleDesc tupleDescriptor, HeapTuple heapTuple)
@@ -351,7 +381,8 @@ TupleToShardPlacement(TupleDesc tupleDescriptor, HeapTuple heapTuple)
 	ShardPlacement *shardPlacement = NULL;
 	bool isNull = false;
 
-	Oid tupleOid = HeapTupleGetOid(heapTuple);
+	Datum placementId = heap_getattr(heapTuple, Anum_pg_dist_shard_placement_placementid,
+									 tupleDescriptor, &isNull);
 	Datum shardId = heap_getattr(heapTuple, Anum_pg_dist_shard_placement_shardid,
 								 tupleDescriptor, &isNull);
 	Datum shardLength = heap_getattr(heapTuple, Anum_pg_dist_shard_placement_shardlength,
@@ -362,11 +393,14 @@ TupleToShardPlacement(TupleDesc tupleDescriptor, HeapTuple heapTuple)
 								  tupleDescriptor, &isNull);
 	Datum nodePort = heap_getattr(heapTuple, Anum_pg_dist_shard_placement_nodeport,
 								  tupleDescriptor, &isNull);
-
-	Assert(!HeapTupleHasNulls(heapTuple));
+	if (HeapTupleHeaderGetNatts(heapTuple->t_data) != Natts_pg_dist_shard_placement ||
+		HeapTupleHasNulls(heapTuple))
+	{
+		ereport(ERROR, (errmsg("unexpected null in pg_dist_shard_placement_tuple")));
+	}
 
 	shardPlacement = CitusMakeNode(ShardPlacement);
-	shardPlacement->tupleOid = tupleOid;
+	shardPlacement->placementId = DatumGetInt64(placementId);
 	shardPlacement->shardId = DatumGetInt64(shardId);
 	shardPlacement->shardLength = DatumGetInt64(shardLength);
 	shardPlacement->shardState = DatumGetUInt32(shardState);
@@ -400,20 +434,19 @@ InsertShardRow(Oid relationId, uint64 shardId, char storageType,
 	values[Anum_pg_dist_shard_shardid - 1] = Int64GetDatum(shardId);
 	values[Anum_pg_dist_shard_shardstorage - 1] = CharGetDatum(storageType);
 
+	/* dropped shardalias column must also be set; it is still part of the tuple */
+	isNulls[Anum_pg_dist_shard_shardalias_DROPPED - 1] = true;
+
 	/* check if shard min/max values are null */
 	if (shardMinValue != NULL && shardMaxValue != NULL)
 	{
 		values[Anum_pg_dist_shard_shardminvalue - 1] = PointerGetDatum(shardMinValue);
 		values[Anum_pg_dist_shard_shardmaxvalue - 1] = PointerGetDatum(shardMaxValue);
-
-		/* we always set shard alias to null */
-		isNulls[Anum_pg_dist_shard_shardalias - 1] = true;
 	}
 	else
 	{
 		isNulls[Anum_pg_dist_shard_shardminvalue - 1] = true;
 		isNulls[Anum_pg_dist_shard_shardmaxvalue - 1] = true;
-		isNulls[Anum_pg_dist_shard_shardalias - 1] = true;
 	}
 
 	/* open shard relation and insert new tuple */
@@ -424,20 +457,23 @@ InsertShardRow(Oid relationId, uint64 shardId, char storageType,
 
 	simple_heap_insert(pgDistShard, heapTuple);
 	CatalogUpdateIndexes(pgDistShard, heapTuple);
-	CommandCounterIncrement();
 
-	/* close relation and invalidate previous cache entry */
-	heap_close(pgDistShard, RowExclusiveLock);
+	/* invalidate previous cache entry and close relation */
 	CitusInvalidateRelcacheByRelid(relationId);
+
+	CommandCounterIncrement();
+	heap_close(pgDistShard, RowExclusiveLock);
 }
 
 
 /*
  * InsertShardPlacementRow opens the shard placement system catalog, and inserts
- * a new row with the given values into that system catalog.
+ * a new row with the given values into that system catalog. If placementId is
+ * INVALID_PLACEMENT_ID, a new placement id will be assigned.
  */
 void
-InsertShardPlacementRow(uint64 shardId, char shardState, uint64 shardLength,
+InsertShardPlacementRow(uint64 shardId, uint64 placementId,
+						char shardState, uint64 shardLength,
 						char *nodeName, uint32 nodePort)
 {
 	Relation pgDistShardPlacement = NULL;
@@ -450,11 +486,16 @@ InsertShardPlacementRow(uint64 shardId, char shardState, uint64 shardLength,
 	memset(values, 0, sizeof(values));
 	memset(isNulls, false, sizeof(isNulls));
 
+	if (placementId == INVALID_PLACEMENT_ID)
+	{
+		placementId = master_get_new_placementid(NULL);
+	}
 	values[Anum_pg_dist_shard_placement_shardid - 1] = Int64GetDatum(shardId);
 	values[Anum_pg_dist_shard_placement_shardstate - 1] = CharGetDatum(shardState);
 	values[Anum_pg_dist_shard_placement_shardlength - 1] = Int64GetDatum(shardLength);
 	values[Anum_pg_dist_shard_placement_nodename - 1] = CStringGetTextDatum(nodeName);
 	values[Anum_pg_dist_shard_placement_nodeport - 1] = Int64GetDatum(nodePort);
+	values[Anum_pg_dist_shard_placement_placementid - 1] = Int64GetDatum(placementId);
 
 	/* open shard placement relation and insert new tuple */
 	pgDistShardPlacement = heap_open(DistShardPlacementRelationId(), RowExclusiveLock);
@@ -464,9 +505,10 @@ InsertShardPlacementRow(uint64 shardId, char shardState, uint64 shardLength,
 
 	simple_heap_insert(pgDistShardPlacement, heapTuple);
 	CatalogUpdateIndexes(pgDistShardPlacement, heapTuple);
-	CommandCounterIncrement();
 
-	/* close relation */
+	CitusInvalidateRelcacheByShardId(shardId);
+
+	CommandCounterIncrement();
 	heap_close(pgDistShardPlacement, RowExclusiveLock);
 }
 
@@ -507,13 +549,14 @@ DeleteShardRow(uint64 shardId)
 	distributedRelationId = pgDistShardForm->logicalrelid;
 
 	simple_heap_delete(pgDistShard, &heapTuple->t_self);
-	CommandCounterIncrement();
 
 	systable_endscan(scanDescriptor);
-	heap_close(pgDistShard, RowExclusiveLock);
 
 	/* invalidate previous cache entry */
 	CitusInvalidateRelcacheByRelid(distributedRelationId);
+
+	CommandCounterIncrement();
+	heap_close(pgDistShard, RowExclusiveLock);
 }
 
 
@@ -522,7 +565,7 @@ DeleteShardRow(uint64 shardId)
  * first (unique) row that corresponds to the given shardId and worker node, and
  * deletes this row.
  */
-void
+uint64
 DeleteShardPlacementRow(uint64 shardId, char *workerName, uint32 workerPort)
 {
 	Relation pgDistShardPlacement = NULL;
@@ -532,8 +575,12 @@ DeleteShardPlacementRow(uint64 shardId, char *workerName, uint32 workerPort)
 	bool indexOK = true;
 	HeapTuple heapTuple = NULL;
 	bool heapTupleFound = false;
+	TupleDesc tupleDescriptor = NULL;
+	int64 placementId = INVALID_PLACEMENT_ID;
+	bool isNull = false;
 
 	pgDistShardPlacement = heap_open(DistShardPlacementRelationId(), RowExclusiveLock);
+	tupleDescriptor = RelationGetDescr(pgDistShardPlacement);
 
 	ScanKeyInit(&scanKey[0], Anum_pg_dist_shard_placement_shardid,
 				BTEqualStrategyNumber, F_INT8EQ, Int64GetDatum(shardId));
@@ -545,8 +592,6 @@ DeleteShardPlacementRow(uint64 shardId, char *workerName, uint32 workerPort)
 	heapTuple = systable_getnext(scanDescriptor);
 	while (HeapTupleIsValid(heapTuple))
 	{
-		TupleDesc tupleDescriptor = RelationGetDescr(pgDistShardPlacement);
-
 		ShardPlacement *placement = TupleToShardPlacement(tupleDescriptor, heapTuple);
 		if (strncmp(placement->nodeName, workerName, WORKER_LENGTH) == 0 &&
 			placement->nodePort == workerPort)
@@ -566,58 +611,84 @@ DeleteShardPlacementRow(uint64 shardId, char *workerName, uint32 workerPort)
 							   shardId, workerName, workerPort)));
 	}
 
-	simple_heap_delete(pgDistShardPlacement, &heapTuple->t_self);
-	CommandCounterIncrement();
+	placementId = heap_getattr(heapTuple, Anum_pg_dist_shard_placement_placementid,
+							   tupleDescriptor, &isNull);
+	if (HeapTupleHeaderGetNatts(heapTuple->t_data) != Natts_pg_dist_shard_placement ||
+		HeapTupleHasNulls(heapTuple))
+	{
+		ereport(ERROR, (errmsg("unexpected null in pg_dist_shard_placement_tuple")));
+	}
 
+	simple_heap_delete(pgDistShardPlacement, &heapTuple->t_self);
 	systable_endscan(scanDescriptor);
+
+	CitusInvalidateRelcacheByShardId(shardId);
+
+	CommandCounterIncrement();
 	heap_close(pgDistShardPlacement, RowExclusiveLock);
+
+	return placementId;
 }
 
 
 /*
- * BuildDistributionKeyFromColumnName builds a simple distribution key consisting
- * only out of a reference to the column of name columnName. Errors out if the
- * specified column does not exist or is not suitable to be used as a
- * distribution column.
+ * UpdateShardPlacementState sets the shardState for the placement identified
+ * by placementId.
  */
-Node *
-BuildDistributionKeyFromColumnName(Relation distributedRelation, char *columnName)
+void
+UpdateShardPlacementState(uint64 placementId, char shardState)
 {
-	HeapTuple columnTuple = NULL;
-	Form_pg_attribute columnForm = NULL;
-	Var *column = NULL;
-	char *tableName = RelationGetRelationName(distributedRelation);
+	Relation pgDistShardPlacement = NULL;
+	SysScanDesc scanDescriptor = NULL;
+	ScanKeyData scanKey[1];
+	int scanKeyCount = 1;
+	bool indexOK = true;
+	HeapTuple heapTuple = NULL;
+	TupleDesc tupleDescriptor = NULL;
+	Datum values[Natts_pg_dist_shard_placement];
+	bool isnull[Natts_pg_dist_shard_placement];
+	bool replace[Natts_pg_dist_shard_placement];
+	uint64 shardId = INVALID_SHARD_ID;
+	bool colIsNull = false;
 
-	/* it'd probably better to downcase identifiers consistent with SQL case folding */
-	truncate_identifier(columnName, strlen(columnName), true);
+	pgDistShardPlacement = heap_open(DistShardPlacementRelationId(), RowExclusiveLock);
+	tupleDescriptor = RelationGetDescr(pgDistShardPlacement);
+	ScanKeyInit(&scanKey[0], Anum_pg_dist_shard_placement_placementid,
+				BTEqualStrategyNumber, F_INT8EQ, Int64GetDatum(placementId));
 
-	/* lookup column definition */
-	columnTuple = SearchSysCacheAttName(RelationGetRelid(distributedRelation),
-										columnName);
-	if (!HeapTupleIsValid(columnTuple))
+	scanDescriptor = systable_beginscan(pgDistShardPlacement,
+										DistShardPlacementPlacementidIndexId(), indexOK,
+										NULL, scanKeyCount, scanKey);
+
+	heapTuple = systable_getnext(scanDescriptor);
+	if (!HeapTupleIsValid(heapTuple))
 	{
-		ereport(ERROR, (errcode(ERRCODE_UNDEFINED_COLUMN),
-						errmsg("column \"%s\" of relation \"%s\" does not exist",
-							   columnName, tableName)));
+		ereport(ERROR, (errmsg("could not find valid entry for shard placement "
+							   UINT64_FORMAT,
+							   placementId)));
 	}
 
-	columnForm = (Form_pg_attribute) GETSTRUCT(columnTuple);
+	memset(replace, 0, sizeof(replace));
 
-	/* check if the column may be referenced in the distribution key */
-	if (columnForm->attnum <= 0)
-	{
-		ereport(ERROR, (errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
-						errmsg("cannot reference system column \"%s\" in relation \"%s\"",
-							   columnName, tableName)));
-	}
+	values[Anum_pg_dist_shard_placement_shardstate - 1] = CharGetDatum(shardState);
+	isnull[Anum_pg_dist_shard_placement_shardstate - 1] = false;
+	replace[Anum_pg_dist_shard_placement_shardstate - 1] = true;
 
-	/* build Var referencing only the chosen distribution column */
-	column = makeVar(1, columnForm->attnum, columnForm->atttypid,
-					 columnForm->atttypmod, columnForm->attcollation, 0);
+	heapTuple = heap_modify_tuple(heapTuple, tupleDescriptor, values, isnull, replace);
+	simple_heap_update(pgDistShardPlacement, &heapTuple->t_self, heapTuple);
 
-	ReleaseSysCache(columnTuple);
+	CatalogUpdateIndexes(pgDistShardPlacement, heapTuple);
 
-	return (Node *) column;
+	shardId = DatumGetInt64(heap_getattr(heapTuple,
+										 Anum_pg_dist_shard_placement_shardid,
+										 tupleDescriptor, &colIsNull));
+	Assert(!colIsNull);
+	CitusInvalidateRelcacheByShardId(shardId);
+
+	CommandCounterIncrement();
+
+	systable_endscan(scanDescriptor);
+	heap_close(pgDistShardPlacement, NoLock);
 }
 
 
@@ -655,6 +726,20 @@ EnsureTableOwner(Oid relationId)
 }
 
 
+/*
+ * EnsureSuperUser check that the current user is a superuser and errors out if not.
+ */
+void
+EnsureSuperUser(void)
+{
+	if (!superuser())
+	{
+		ereport(ERROR, (errmsg("operation is not allowed"),
+						errhint("Run the command with a superuser.")));
+	}
+}
+
+
 /*
  * Return a table's owner as a string.
  */
@@ -806,7 +891,8 @@ master_stage_shard_placement_row(PG_FUNCTION_ARGS)
 	EnsureTablePermissions(distributedRelationId, ACL_INSERT);
 
 	/* finally insert placement */
-	InsertShardPlacementRow(shardId, shardState, shardLength, nodeName, nodePort);
+	InsertShardPlacementRow(shardId, INVALID_PLACEMENT_ID, shardState, shardLength,
+							nodeName, nodePort);
 
 	heap_close(relation, NoLock);
 	heap_close(pgDistShard, NoLock);
