@@ -31,6 +31,10 @@
 #include "catalog/indexing.h"
 #include "catalog/namespace.h"
 #include "catalog/pg_class.h"
+#include "catalog/pg_constraint.h"
+#if (PG_VERSION_NUM >= 90600)
+#include "catalog/pg_constraint_fn.h"
+#endif
 #include "catalog/pg_index.h"
 #include "catalog/pg_type.h"
 #include "commands/sequence.h"
@@ -55,6 +59,7 @@
 
 
 /* Shard related configuration */
+int ShardCount = 32;
 int ShardReplicationFactor = 2; /* desired replication factor for shards */
 int ShardMaxSize = 1048576;     /* maximum size in KB one shard can grow to */
 int ShardPlacementPolicy = SHARD_PLACEMENT_ROUND_ROBIN;
@@ -67,6 +72,7 @@ static Datum WorkerNodeGetDatum(WorkerNode *workerNode, TupleDesc tupleDescripto
 PG_FUNCTION_INFO_V1(master_get_table_metadata);
 PG_FUNCTION_INFO_V1(master_get_table_ddl_events);
 PG_FUNCTION_INFO_V1(master_get_new_shardid);
+PG_FUNCTION_INFO_V1(master_get_new_placementid);
 PG_FUNCTION_INFO_V1(master_get_local_first_candidate_nodes);
 PG_FUNCTION_INFO_V1(master_get_round_robin_candidate_nodes);
 PG_FUNCTION_INFO_V1(master_get_active_worker_nodes);
@@ -92,8 +98,7 @@ master_get_table_metadata(PG_FUNCTION_ARGS)
 	HeapTuple metadataTuple = NULL;
 	TupleDesc metadataDescriptor = NULL;
 	uint64 shardMaxSizeInBytes = 0;
-	char relationType = 0;
-	char storageType = 0;
+	char shardStorageType = 0;
 	Datum values[TABLE_METADATA_FIELDS];
 	bool isNulls[TABLE_METADATA_FIELDS];
 
@@ -120,26 +125,10 @@ master_get_table_metadata(PG_FUNCTION_ARGS)
 	shardMaxSizeInBytes = (int64) ShardMaxSize * 1024L;
 
 	/* get storage type */
-	relationType = get_rel_relkind(relationId);
-	if (relationType == RELKIND_RELATION)
-	{
-		storageType = SHARD_STORAGE_TABLE;
-	}
-	else if (relationType == RELKIND_FOREIGN_TABLE)
-	{
-		bool cstoreTable = CStoreTable(relationId);
-		if (cstoreTable)
-		{
-			storageType = SHARD_STORAGE_COLUMNAR;
-		}
-		else
-		{
-			storageType = SHARD_STORAGE_FOREIGN;
-		}
-	}
+	shardStorageType = ShardStorageType(relationId);
 
 	values[0] = ObjectIdGetDatum(relationId);
-	values[1] = storageType;
+	values[1] = shardStorageType;
 	values[2] = partitionEntry->partitionMethod;
 	values[3] = partitionKey;
 	values[4] = Int32GetDatum(ShardReplicationFactor);
@@ -246,13 +235,8 @@ master_get_table_ddl_events(PG_FUNCTION_ARGS)
 
 
 /*
- * master_get_new_shardid allocates and returns a unique shardId for the shard
- * to be created. This allocation occurs both in shared memory and in write
- * ahead logs; writing to logs avoids the risk of having shardId collisions.
- *
- * Please note that the caller is still responsible for finalizing shard data
- * and the shardId with the master node. Further note that this function relies
- * on an internal sequence created in initdb to generate unique identifiers.
+ * master_get_new_shardid is a user facing wrapper function around GetNextShardId()
+ * which allocates and returns a unique shardId for the shard to be created.
  *
  * NB: This can be called by any user; for now we have decided that that's
  * ok. We might want to restrict this to users part of a specific role or such
@@ -260,6 +244,24 @@ master_get_table_ddl_events(PG_FUNCTION_ARGS)
  */
 Datum
 master_get_new_shardid(PG_FUNCTION_ARGS)
+{
+	uint64 shardId = GetNextShardId();
+	Datum shardIdDatum = Int64GetDatum(shardId);
+
+	PG_RETURN_DATUM(shardIdDatum);
+}
+
+
+/*
+ * GetNextShardId allocates and returns a unique shardId for the shard to be
+ * created. This allocation occurs both in shared memory and in write ahead
+ * logs; writing to logs avoids the risk of having shardId collisions.
+ *
+ * Please note that the caller is still responsible for finalizing shard data
+ * and the shardId with the master node.
+ */
+uint64
+GetNextShardId()
 {
 	text *sequenceName = cstring_to_text(SHARDID_SEQUENCE_NAME);
 	Oid sequenceId = ResolveRelationId(sequenceName);
@@ -267,6 +269,41 @@ master_get_new_shardid(PG_FUNCTION_ARGS)
 	Oid savedUserId = InvalidOid;
 	int savedSecurityContext = 0;
 	Datum shardIdDatum = 0;
+	uint64 shardId = 0;
+
+	GetUserIdAndSecContext(&savedUserId, &savedSecurityContext);
+	SetUserIdAndSecContext(CitusExtensionOwner(), SECURITY_LOCAL_USERID_CHANGE);
+
+	/* generate new and unique shardId from sequence */
+	shardIdDatum = DirectFunctionCall1(nextval_oid, sequenceIdDatum);
+
+	SetUserIdAndSecContext(savedUserId, savedSecurityContext);
+
+	shardId = DatumGetInt64(shardIdDatum);
+
+	return shardId;
+}
+
+
+/*
+ * master_get_new_placementid allocates and returns a unique placementId for
+ * the placement to be created. This allocation occurs both in shared memory
+ * and in write ahead logs; writing to logs avoids the risk of having shardId
+ * collisions.
+ *
+ * NB: This can be called by any user; for now we have decided that that's
+ * ok. We might want to restrict this to users part of a specific role or such
+ * at some later point.
+ */
+Datum
+master_get_new_placementid(PG_FUNCTION_ARGS)
+{
+	text *sequenceName = cstring_to_text(PLACEMENTID_SEQUENCE_NAME);
+	Oid sequenceId = ResolveRelationId(sequenceName);
+	Datum sequenceIdDatum = ObjectIdGetDatum(sequenceId);
+	Oid savedUserId = InvalidOid;
+	int savedSecurityContext = 0;
+	Datum shardIdDatum = 0;
 
 	GetUserIdAndSecContext(&savedUserId, &savedSecurityContext);
 	SetUserIdAndSecContext(CitusExtensionOwner(), SECURITY_LOCAL_USERID_CHANGE);
@@ -459,7 +496,7 @@ master_get_round_robin_candidate_nodes(PG_FUNCTION_ARGS)
 /*
  * master_get_active_worker_nodes returns a set of active worker host names and
  * port numbers in deterministic order. Currently we assume that all worker
- * nodes in pg_worker_list.conf are active.
+ * nodes in pg_dist_node are active.
  */
 Datum
 master_get_active_worker_nodes(PG_FUNCTION_ARGS)
@@ -546,7 +583,7 @@ ResolveRelationId(text *relationName)
  * GetTableDDLEvents takes in a relationId, and returns the list of DDL commands
  * needed to reconstruct the relation. These DDL commands are all palloced; and
  * include the table's schema definition, optional column storage and statistics
- * definitions, and index and constraint defitions.
+ * definitions, and index and constraint definitions.
  */
 List *
 GetTableDDLEvents(Oid relationId)
@@ -640,15 +677,15 @@ GetTableDDLEvents(Oid relationId)
 
 		/*
 		 * A primary key index is always created by a constraint statement.
-		 * A unique key index is created by a constraint if and only if the
-		 * index has a corresponding constraint entry in pg_depend. Any other
-		 * index form is never associated with a constraint.
+		 * A unique key index or exclusion index is created by a constraint
+		 * if and only if the index has a corresponding constraint entry in pg_depend.
+		 * Any other index form is never associated with a constraint.
 		 */
 		if (indexForm->indisprimary)
 		{
 			isConstraint = true;
 		}
-		else if (indexForm->indisunique)
+		else if (indexForm->indisunique || indexForm->indisexclusion)
 		{
 			Oid constraintId = get_index_constraint(indexId);
 			isConstraint = OidIsValid(constraintId);
@@ -697,6 +734,102 @@ GetTableDDLEvents(Oid relationId)
 }
 
 
+/*
+ * GetTableForeignConstraints takes in a relationId, and returns the list of foreign
+ * constraint commands needed to reconstruct foreign constraints of that table.
+ */
+List *
+GetTableForeignConstraintCommands(Oid relationId)
+{
+	List *tableForeignConstraints = NIL;
+
+	Relation pgConstraint = NULL;
+	SysScanDesc scanDescriptor = NULL;
+	ScanKeyData scanKey[1];
+	int scanKeyCount = 1;
+	HeapTuple heapTuple = NULL;
+
+	/*
+	 * Set search_path to NIL so that all objects outside of pg_catalog will be
+	 * schema-prefixed. pg_catalog will be added automatically when we call
+	 * PushOverrideSearchPath(), since we set addCatalog to true;
+	 */
+	OverrideSearchPath *overridePath = GetOverrideSearchPath(CurrentMemoryContext);
+	overridePath->schemas = NIL;
+	overridePath->addCatalog = true;
+	PushOverrideSearchPath(overridePath);
+
+	/* open system catalog and scan all constraints that belong to this table */
+	pgConstraint = heap_open(ConstraintRelationId, AccessShareLock);
+	ScanKeyInit(&scanKey[0], Anum_pg_constraint_conrelid, BTEqualStrategyNumber, F_OIDEQ,
+				relationId);
+	scanDescriptor = systable_beginscan(pgConstraint, ConstraintRelidIndexId, true, NULL,
+										scanKeyCount, scanKey);
+
+	heapTuple = systable_getnext(scanDescriptor);
+	while (HeapTupleIsValid(heapTuple))
+	{
+		Form_pg_constraint constraintForm = (Form_pg_constraint) GETSTRUCT(heapTuple);
+
+		if (constraintForm->contype == CONSTRAINT_FOREIGN)
+		{
+			Oid constraintId = get_relation_constraint_oid(relationId,
+														   constraintForm->conname.data,
+														   true);
+			char *statementDef = pg_get_constraintdef_command(constraintId);
+
+			tableForeignConstraints = lappend(tableForeignConstraints, statementDef);
+		}
+
+		heapTuple = systable_getnext(scanDescriptor);
+	}
+
+	/* clean up scan and close system catalog */
+	systable_endscan(scanDescriptor);
+	heap_close(pgConstraint, AccessShareLock);
+
+	/* revert back to original search_path */
+	PopOverrideSearchPath();
+
+	return tableForeignConstraints;
+}
+
+
+/*
+ * ShardStorageType returns the shard storage type according to relation type.
+ */
+char
+ShardStorageType(Oid relationId)
+{
+	char shardStorageType = 0;
+
+	char relationType = get_rel_relkind(relationId);
+	if (relationType == RELKIND_RELATION)
+	{
+		shardStorageType = SHARD_STORAGE_TABLE;
+	}
+	else if (relationType == RELKIND_FOREIGN_TABLE)
+	{
+		bool cstoreTable = CStoreTable(relationId);
+		if (cstoreTable)
+		{
+			shardStorageType = SHARD_STORAGE_COLUMNAR;
+		}
+		else
+		{
+			shardStorageType = SHARD_STORAGE_FOREIGN;
+		}
+	}
+	else
+	{
+		ereport(ERROR, (errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
+						errmsg("unexpected relation type: %c", relationType)));
+	}
+
+	return shardStorageType;
+}
+
+
 /*
  * WorkerNodeGetDatum converts the worker node passed to it into its datum
  * representation. To do this, the function first creates the heap tuple from
