@@ -20,7 +20,10 @@
 
 #include "access/htup_details.h"
 #include "access/xact.h"
+#include "commands/tablecmds.h"
 #include "catalog/indexing.h"
+#include "catalog/namespace.h"
+#include "distributed/colocation_utils.h"
 #include "distributed/multi_client_executor.h"
 #include "distributed/master_metadata_utility.h"
 #include "distributed/master_protocol.h"
@@ -41,8 +44,6 @@
 
 
 /* Local functions forward declarations */
-static bool WorkerCreateShard(Oid relationId, char *nodeName, uint32 nodePort,
-							  uint64 shardId, char *newShardOwner, List *ddlCommandList);
 static bool WorkerShardStats(char *nodeName, uint32 nodePort, Oid relationId,
 							 char *shardName, uint64 *shardSize,
 							 text **shardMinValue, text **shardMaxValue);
@@ -66,8 +67,7 @@ master_create_empty_shard(PG_FUNCTION_ARGS)
 	text *relationNameText = PG_GETARG_TEXT_P(0);
 	char *relationName = text_to_cstring(relationNameText);
 	List *workerNodeList = WorkerNodeList();
-	Datum shardIdDatum = 0;
-	int64 shardId = INVALID_SHARD_ID;
+	uint64 shardId = INVALID_SHARD_ID;
 	List *ddlEventList = NULL;
 	uint32 attemptableNodeCount = 0;
 	uint32 liveNodeCount = 0;
@@ -116,8 +116,7 @@ master_create_empty_shard(PG_FUNCTION_ARGS)
 	}
 
 	/* generate new and unique shardId from sequence */
-	shardIdDatum = master_get_new_shardid(NULL);
-	shardId = DatumGetInt64(shardIdDatum);
+	shardId = GetNextShardId();
 
 	/* get table DDL commands to replay on the worker node */
 	ddlEventList = GetTableDDLEvents(relationId);
@@ -238,13 +237,9 @@ master_append_table_to_shard(PG_FUNCTION_ARGS)
 	shardSchemaOid = get_rel_namespace(relationId);
 	shardSchemaName = get_namespace_name(shardSchemaOid);
 
-	/* if shard doesn't have an alias, extend regular table name */
-	shardTableName = LoadShardAlias(relationId, shardId);
-	if (shardTableName == NULL)
-	{
-		shardTableName = get_rel_name(relationId);
-		AppendShardIdToName(&shardTableName, shardId);
-	}
+	/* Build shard table name. */
+	shardTableName = get_rel_name(relationId);
+	AppendShardIdToName(&shardTableName, shardId);
 
 	shardQualifiedName = quote_qualified_identifier(shardSchemaName, shardTableName);
 
@@ -296,13 +291,11 @@ master_append_table_to_shard(PG_FUNCTION_ARGS)
 	foreach(failedPlacementCell, failedPlacementList)
 	{
 		ShardPlacement *placement = (ShardPlacement *) lfirst(failedPlacementCell);
+		uint64 placementId = placement->placementId;
 		char *workerName = placement->nodeName;
 		uint32 workerPort = placement->nodePort;
-		uint64 oldShardLength = placement->shardLength;
 
-		DeleteShardPlacementRow(shardId, workerName, workerPort);
-		InsertShardPlacementRow(shardId, FILE_INACTIVE, oldShardLength,
-								workerName, workerPort);
+		UpdateShardPlacementState(placementId, FILE_INACTIVE);
 
 		ereport(WARNING, (errmsg("could not append table to shard \"%s\" on node "
 								 "\"%s:%u\"", shardQualifiedName, workerName,
@@ -394,15 +387,33 @@ CreateShardPlacements(Oid relationId, int64 shardId, List *ddlEventList,
 		WorkerNode *workerNode = (WorkerNode *) list_nth(workerNodeList, workerNodeIndex);
 		char *nodeName = workerNode->workerName;
 		uint32 nodePort = workerNode->workerPort;
+		List *foreignConstraintCommandList = GetTableForeignConstraintCommands(
+			relationId);
+		int shardIndex = -1; /* not used in this code path */
+		bool created = false;
 
-		bool created = WorkerCreateShard(relationId, nodeName, nodePort, shardId,
-										 newPlacementOwner, ddlEventList);
+		/*
+		 * In this code path, we create not co-located tables. Therefore we should error
+		 * out if there is a foreign key constraint specified.
+		 */
+		if (foreignConstraintCommandList != NIL)
+		{
+			ereport(ERROR, (errmsg("could only create distributed table"),
+							errdetail("Table contains foreign key constraints. Foreign "
+									  "key constraints only supported with co-located "
+									  "tables")));
+		}
+
+		created = WorkerCreateShard(relationId, nodeName, nodePort, shardIndex,
+									shardId, newPlacementOwner, ddlEventList,
+									foreignConstraintCommandList);
 		if (created)
 		{
 			const RelayFileState shardState = FILE_FINALIZED;
 			const uint64 shardSize = 0;
 
-			InsertShardPlacementRow(shardId, shardState, shardSize, nodeName, nodePort);
+			InsertShardPlacementRow(shardId, INVALID_PLACEMENT_ID, shardState, shardSize,
+									nodeName, nodePort);
 			placementsCreated++;
 		}
 		else
@@ -431,14 +442,17 @@ CreateShardPlacements(Oid relationId, int64 shardId, List *ddlEventList,
  * shard on the worker node. Note that this function opens a new connection for
  * each DDL command, and could leave the shard in an half-initialized state.
  */
-static bool
+bool
 WorkerCreateShard(Oid relationId, char *nodeName, uint32 nodePort,
-				  uint64 shardId, char *newShardOwner, List *ddlCommandList)
+				  int shardIndex, uint64 shardId, char *newShardOwner,
+				  List *ddlCommandList, List *foreignConstraintCommandList)
 {
 	Oid schemaId = get_rel_namespace(relationId);
 	char *schemaName = get_namespace_name(schemaId);
+	char *escapedSchemaName = quote_literal_cstr(schemaName);
 	bool shardCreated = true;
 	ListCell *ddlCommandCell = NULL;
+	ListCell *foreignConstraintCommandCell = NULL;
 
 	foreach(ddlCommandCell, ddlCommandList)
 	{
@@ -449,8 +463,6 @@ WorkerCreateShard(Oid relationId, char *nodeName, uint32 nodePort,
 
 		if (strcmp(schemaName, "public") != 0)
 		{
-			char *escapedSchemaName = quote_literal_cstr(schemaName);
-
 			appendStringInfo(applyDDLCommand, WORKER_APPLY_SHARD_DDL_COMMAND, shardId,
 							 escapedSchemaName, escapedDDLCommand);
 		}
@@ -470,6 +482,47 @@ WorkerCreateShard(Oid relationId, char *nodeName, uint32 nodePort,
 		}
 	}
 
+	foreach(foreignConstraintCommandCell, foreignConstraintCommandList)
+	{
+		char *command = (char *) lfirst(foreignConstraintCommandCell);
+		char *escapedCommand = quote_literal_cstr(command);
+
+		Oid referencedRelationId = InvalidOid;
+		Oid referencedSchemaId = InvalidOid;
+		char *referencedSchemaName = NULL;
+		char *escapedReferencedSchemaName = NULL;
+		uint64 referencedShardId = INVALID_SHARD_ID;
+
+		List *queryResultList = NIL;
+		StringInfo applyForeignConstraintCommand = makeStringInfo();
+
+		/* we need to parse the foreign constraint command to get referencing table id */
+		referencedRelationId = ForeignConstraintGetReferencedTableId(command);
+		if (referencedRelationId == InvalidOid)
+		{
+			ereport(ERROR, (errcode(ERRCODE_INVALID_PARAMETER_VALUE),
+							errmsg("cannot create foreign key constraint"),
+							errdetail("Referenced relation cannot be found.")));
+		}
+
+		referencedSchemaId = get_rel_namespace(referencedRelationId);
+		referencedSchemaName = get_namespace_name(referencedSchemaId);
+		escapedReferencedSchemaName = quote_literal_cstr(referencedSchemaName);
+		referencedShardId = ColocatedShardIdInRelation(referencedRelationId, shardIndex);
+
+		appendStringInfo(applyForeignConstraintCommand,
+						 WORKER_APPLY_INTER_SHARD_DDL_COMMAND, shardId, escapedSchemaName,
+						 referencedShardId, escapedReferencedSchemaName, escapedCommand);
+
+		queryResultList = ExecuteRemoteQuery(nodeName, nodePort, newShardOwner,
+											 applyForeignConstraintCommand);
+		if (queryResultList == NIL)
+		{
+			shardCreated = false;
+			break;
+		}
+	}
+
 	return shardCreated;
 }
 
@@ -493,19 +546,14 @@ UpdateShardStatistics(int64 shardId)
 	text *minValue = NULL;
 	text *maxValue = NULL;
 
-	/* if shard doesn't have an alias, extend regular table name */
-	shardQualifiedName = LoadShardAlias(relationId, shardId);
-	if (shardQualifiedName == NULL)
-	{
-		char *shardName = get_rel_name(relationId);
+	/* Build shard qualified name. */
+	char *shardName = get_rel_name(relationId);
+	Oid schemaId = get_rel_namespace(relationId);
+	char *schemaName = get_namespace_name(schemaId);
 
-		Oid schemaId = get_rel_namespace(relationId);
-		char *schemaName = get_namespace_name(schemaId);
+	AppendShardIdToName(&shardName, shardId);
 
-		AppendShardIdToName(&shardName, shardId);
-
-		shardQualifiedName = quote_qualified_identifier(schemaName, shardName);
-	}
+	shardQualifiedName = quote_qualified_identifier(schemaName, shardName);
 
 	shardPlacementList = FinalizedShardPlacementList(shardId);
 
@@ -546,11 +594,12 @@ UpdateShardStatistics(int64 shardId)
 	foreach(shardPlacementCell, shardPlacementList)
 	{
 		ShardPlacement *placement = (ShardPlacement *) lfirst(shardPlacementCell);
+		uint64 placementId = placement->placementId;
 		char *workerName = placement->nodeName;
 		uint32 workerPort = placement->nodePort;
 
 		DeleteShardPlacementRow(shardId, workerName, workerPort);
-		InsertShardPlacementRow(shardId, FILE_FINALIZED, shardSize,
+		InsertShardPlacementRow(shardId, placementId, FILE_FINALIZED, shardSize,
 								workerName, workerPort);
 	}
 
@@ -695,3 +744,31 @@ WorkerShardStats(char *nodeName, uint32 nodePort, Oid relationId, char *shardNam
 
 	return true;
 }
+
+
+/*
+ * ForeignConstraintGetReferencedTableId parses given foreign constraint query and
+ * extracts referenced table id from it.
+ */
+Oid
+ForeignConstraintGetReferencedTableId(char *queryString)
+{
+	Node *queryNode = ParseTreeNode(queryString);
+	AlterTableStmt *foreignConstraintStmt = (AlterTableStmt *) queryNode;
+	AlterTableCmd *command = (AlterTableCmd *) linitial(foreignConstraintStmt->cmds);
+
+	if (command->subtype == AT_AddConstraint)
+	{
+		Constraint *constraint = (Constraint *) command->def;
+		if (constraint->contype == CONSTR_FOREIGN)
+		{
+			RangeVar *referencedTable = constraint->pktable;
+			LOCKMODE lockmode = AlterTableGetLockLevel(foreignConstraintStmt->cmds);
+
+			return RangeVarGetRelid(referencedTable, lockmode,
+									foreignConstraintStmt->missing_ok);
+		}
+	}
+
+	return InvalidOid;
+}
