@@ -642,7 +642,8 @@ CitusHasBeenLoaded(void)
 			}
 		}
 
-		extensionLoaded = extensionPresent && extensionScriptExecuted;
+		/* we disable extension features during pg_upgrade */
+		extensionLoaded = extensionPresent && extensionScriptExecuted && !IsBinaryUpgrade;
 
 		if (extensionLoaded)
 		{
@@ -653,6 +654,9 @@ CitusHasBeenLoaded(void)
 			 *
 			 * Ensure InvalidateDistRelationCacheCallback will notice those changes
 			 * by caching pg_dist_partition's oid.
+			 *
+			 * We skip these checks during upgrade since pg_dist_partition is not
+			 * present during early stages of upgrade operation.
 			 */
 			DistPartitionRelationId();
 		}
