@@ -74,6 +74,7 @@ static bool check_log_statement(List *stmt_list);
 PG_FUNCTION_INFO_V1(worker_fetch_partition_file);
 PG_FUNCTION_INFO_V1(worker_fetch_query_results_file);
 PG_FUNCTION_INFO_V1(worker_apply_shard_ddl_command);
+PG_FUNCTION_INFO_V1(worker_apply_inter_shard_ddl_command);
 PG_FUNCTION_INFO_V1(worker_fetch_regular_table);
 PG_FUNCTION_INFO_V1(worker_fetch_foreign_file);
 PG_FUNCTION_INFO_V1(worker_append_table_to_shard);
@@ -418,6 +419,36 @@ worker_apply_shard_ddl_command(PG_FUNCTION_ARGS)
 }
 
 
+/*
+ * worker_apply_inter_shard_ddl_command extends table, index, or constraint names in
+ * the given DDL command. The function then applies this extended DDL command
+ * against the database.
+ */
+Datum
+worker_apply_inter_shard_ddl_command(PG_FUNCTION_ARGS)
+{
+	uint64 leftShardId = PG_GETARG_INT64(0);
+	text *leftShardSchemaNameText = PG_GETARG_TEXT_P(1);
+	uint64 rightShardId = PG_GETARG_INT64(2);
+	text *rightShardSchemaNameText = PG_GETARG_TEXT_P(3);
+	text *ddlCommandText = PG_GETARG_TEXT_P(4);
+
+	char *leftShardSchemaName = text_to_cstring(leftShardSchemaNameText);
+	char *rightShardSchemaName = text_to_cstring(rightShardSchemaNameText);
+	const char *ddlCommand = text_to_cstring(ddlCommandText);
+	Node *ddlCommandNode = ParseTreeNode(ddlCommand);
+
+	/* extend names in ddl command and apply extended command */
+	RelayEventExtendNamesForInterShardCommands(ddlCommandNode, leftShardId,
+											   leftShardSchemaName, rightShardId,
+											   rightShardSchemaName);
+	ProcessUtility(ddlCommandNode, ddlCommand, PROCESS_UTILITY_TOPLEVEL, NULL,
+				   None_Receiver, NULL);
+
+	PG_RETURN_VOID();
+}
+
+
 /*
  * worker_fetch_regular_table caches the given PostgreSQL table on the local
  * node. The function caches this table by trying the given list of node names
@@ -1002,6 +1033,65 @@ ExecuteRemoteQuery(const char *nodeName, uint32 nodePort, char *runAsUser,
 }
 
 
+/*
+ * ExecuteRemoteCommand executes the given SQL command. This command could be an
+ * Insert, Update, or Delete statement, or a utility command that returns
+ * nothing. If query is successfuly executed, the function returns true.
+ * Otherwise, it returns false.
+ */
+bool
+ExecuteRemoteCommand(const char *nodeName, uint32 nodePort, StringInfo queryString)
+{
+	char *nodeDatabase = get_database_name(MyDatabaseId);
+	int32 connectionId = -1;
+	QueryStatus queryStatus = CLIENT_INVALID_QUERY;
+	bool querySent = false;
+	bool queryReady = false;
+	bool queryDone = false;
+
+	connectionId = MultiClientConnect(nodeName, nodePort, nodeDatabase, NULL);
+	if (connectionId == INVALID_CONNECTION_ID)
+	{
+		return false;
+	}
+
+	querySent = MultiClientSendQuery(connectionId, queryString->data);
+	if (!querySent)
+	{
+		MultiClientDisconnect(connectionId);
+		return false;
+	}
+
+	while (!queryReady)
+	{
+		ResultStatus resultStatus = MultiClientResultStatus(connectionId);
+		if (resultStatus == CLIENT_RESULT_READY)
+		{
+			queryReady = true;
+		}
+		else if (resultStatus == CLIENT_RESULT_BUSY)
+		{
+			long sleepIntervalPerCycle = RemoteTaskCheckInterval * 1000L;
+			pg_usleep(sleepIntervalPerCycle);
+		}
+		else
+		{
+			MultiClientDisconnect(connectionId);
+			return false;
+		}
+	}
+
+	queryStatus = MultiClientQueryStatus(connectionId);
+	if (queryStatus == CLIENT_QUERY_DONE)
+	{
+		queryDone = true;
+	}
+
+	MultiClientDisconnect(connectionId);
+	return queryDone;
+}
+
+
 /*
  * Parses the given DDL command, and returns the tree node for parsed command.
  */
