@@ -48,8 +48,8 @@
 /* Name of columnar foreign data wrapper */
 #define CSTORE_FDW_NAME "cstore_fdw"
 
-/* ShardId sequence name as defined in initdb.c */
 #define SHARDID_SEQUENCE_NAME "pg_dist_shardid_seq"
+#define PLACEMENTID_SEQUENCE_NAME "pg_dist_shard_placement_placementid_seq"
 
 /* Remote call definitions to help with data staging and deletion */
 #define WORKER_APPLY_SHARD_DDL_COMMAND \
@@ -58,11 +58,14 @@
 	"SELECT worker_apply_shard_ddl_command (" UINT64_FORMAT ", %s)"
 #define WORKER_APPEND_TABLE_TO_SHARD \
 	"SELECT worker_append_table_to_shard (%s, %s, %s, %u)"
+#define WORKER_APPLY_INTER_SHARD_DDL_COMMAND \
+	"SELECT worker_apply_inter_shard_ddl_command (" UINT64_FORMAT ", %s, " UINT64_FORMAT \
+	", %s, %s)"
 #define SHARD_RANGE_QUERY "SELECT min(%s), max(%s) FROM %s"
 #define SHARD_TABLE_SIZE_QUERY "SELECT pg_table_size(%s)"
 #define SHARD_CSTORE_TABLE_SIZE_QUERY "SELECT cstore_table_size(%s)"
-#define DROP_REGULAR_TABLE_COMMAND "DROP TABLE IF EXISTS %s"
-#define DROP_FOREIGN_TABLE_COMMAND "DROP FOREIGN TABLE IF EXISTS %s"
+#define DROP_REGULAR_TABLE_COMMAND "DROP TABLE IF EXISTS %s CASCADE"
+#define DROP_FOREIGN_TABLE_COMMAND "DROP FOREIGN TABLE IF EXISTS %s CASCADE"
 #define CREATE_SCHEMA_COMMAND "CREATE SCHEMA IF NOT EXISTS %s"
 #define CREATE_EMPTY_SHARD_QUERY "SELECT master_create_empty_shard('%s')"
 #define FINALIZED_SHARD_PLACEMENTS_QUERY \
@@ -82,6 +85,7 @@ typedef enum
 
 
 /* Config variables managed via guc.c */
+extern int ShardCount;
 extern int ShardReplicationFactor;
 extern int ShardMaxSize;
 extern int ShardPlacementPolicy;
@@ -89,18 +93,30 @@ extern int ShardPlacementPolicy;
 
 /* Function declarations local to the distributed module */
 extern bool CStoreTable(Oid relationId);
+extern uint64 GetNextShardId(void);
 extern Oid ResolveRelationId(text *relationName);
 extern List * GetTableDDLEvents(Oid relationId);
+extern List * GetTableForeignConstraintCommands(Oid relationId);
+extern char ShardStorageType(Oid relationId);
 extern void CheckDistributedTable(Oid relationId);
 extern void CreateShardPlacements(Oid relationId, int64 shardId, List *ddlEventList,
 								  char *newPlacementOwner, List *workerNodeList,
 								  int workerStartIndex, int replicationFactor);
 extern uint64 UpdateShardStatistics(int64 shardId);
+extern void CreateShardsWithRoundRobinPolicy(Oid distributedTableId, int32 shardCount,
+											 int32 replicationFactor);
+extern void CreateColocatedShards(Oid targetRelationId, Oid sourceRelationId);
+extern bool WorkerCreateShard(Oid relationId, char *nodeName, uint32 nodePort,
+							  int shardIndex, uint64 shardId, char *newShardOwner,
+							  List *ddlCommandList, List *foreignConstraintCommadList);
+extern Oid ForeignConstraintGetReferencedTableId(char *queryString);
+extern void CheckHashPartitionedTable(Oid distributedTableId);
 
-/* Function declarations for generating metadata for shard creation */
+/* Function declarations for generating metadata for shard and placement creation */
 extern Datum master_get_table_metadata(PG_FUNCTION_ARGS);
 extern Datum master_get_table_ddl_events(PG_FUNCTION_ARGS);
 extern Datum master_get_new_shardid(PG_FUNCTION_ARGS);
+extern Datum master_get_new_placementid(PG_FUNCTION_ARGS);
 extern Datum master_get_local_first_candidate_nodes(PG_FUNCTION_ARGS);
 extern Datum master_get_round_robin_candidate_nodes(PG_FUNCTION_ARGS);
 extern Datum master_get_active_worker_nodes(PG_FUNCTION_ARGS);
