@@ -14,6 +14,8 @@
 #include "fmgr.h"
 #include "distributed/master_metadata_utility.h"
 #include "distributed/pg_dist_partition.h"
+#include "distributed/worker_manager.h"
+#include "utils/hsearch.h"
 
 
 /*
@@ -38,6 +40,8 @@ typedef struct
 	/* pg_dist_partition metadata for this table */
 	char *partitionKeyString;
 	char partitionMethod;
+	uint32 colocationId;
+	char replicationModel;
 
 	/* pg_dist_shard metadata (variable-length ShardInterval array) for this table */
 	int shardIntervalArrayLength;
@@ -49,27 +53,44 @@ typedef struct
 
 
 extern bool IsDistributedTable(Oid relationId);
+extern List * DistributedTableList(void);
 extern ShardInterval * LoadShardInterval(uint64 shardId);
 extern DistTableCacheEntry * DistributedTableCacheEntry(Oid distributedRelationId);
+extern int GetLocalGroupId(void);
 extern void CitusInvalidateRelcacheByRelid(Oid relationId);
+extern void CitusInvalidateRelcacheByShardId(int64 shardId);
+extern void CitusInvalidateNodeCache(void);
 
 extern bool CitusHasBeenLoaded(void);
 
+/* access WorkerNodeHash */
+extern HTAB * GetWorkerNodeHash(void);
+
 /* relation oids */
+extern Oid DistColocationRelationId(void);
+extern Oid DistColocationConfigurationIndexId(void);
 extern Oid DistPartitionRelationId(void);
 extern Oid DistShardRelationId(void);
 extern Oid DistShardPlacementRelationId(void);
+extern Oid DistNodeRelationId(void);
+extern Oid DistLocalGroupIdRelationId(void);
 
 /* index oids */
 extern Oid DistPartitionLogicalRelidIndexId(void);
+extern Oid DistPartitionColocationidIndexId(void);
 extern Oid DistShardLogicalRelidIndexId(void);
 extern Oid DistShardShardidIndexId(void);
 extern Oid DistShardPlacementShardidIndexId(void);
+extern Oid DistShardPlacementPlacementidIndexId(void);
+extern Oid DistTransactionRelationId(void);
+extern Oid DistTransactionGroupIndexId(void);
+extern Oid DistShardPlacementNodeidIndexId(void);
 
 /* function oids */
 extern Oid CitusExtraDataContainerFuncId(void);
 
 /* user related functions */
 extern Oid CitusExtensionOwner(void);
+extern char * CitusExtensionOwnerName(void);
 extern char * CurrentUserName(void);
 #endif /* METADATA_CACHE_H */
