@@ -35,7 +35,7 @@
  */
 typedef struct MultiNode
 {
-	CitusNodeTag type;
+	CitusNode type;
 
 	struct MultiNode *parentNode;
 
@@ -171,6 +171,7 @@ typedef struct MultiExtendedOp
 	List *sortClauseList;
 	Node *limitCount;
 	Node *limitOffset;
+	Node *havingQual;
 } MultiExtendedOp;
 
 
