@@ -33,10 +33,12 @@ typedef struct XactShardConnSet
 extern bool AllModificationsCommutative;
 
 
-extern void RouterExecutorStart(QueryDesc *queryDesc, int eflags, Task *task);
+extern void RouterExecutorStart(QueryDesc *queryDesc, int eflags, List *taskList);
 extern void RouterExecutorRun(QueryDesc *queryDesc, ScanDirection direction, long count);
 extern void RouterExecutorFinish(QueryDesc *queryDesc);
 extern void RouterExecutorEnd(QueryDesc *queryDesc);
-extern void InstallRouterExecutorShmemHook(void);
+extern void RegisterRouterExecutorXactCallbacks(void);
+
+extern int64 ExecuteModifyTasksWithoutResults(List *taskList);
 
 #endif /* MULTI_ROUTER_EXECUTOR_H_ */
